/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

if (!QfqNS) {
    var QfqNS = {};
}

if (!QfqNS.Element) {
    QfqNS.Element = {};
}

(function (n) {
    'use strict';

    /**
     * Form Group represents a `<input>/<select>` element including the label and help block.
     *
     * It is not meant to be used directly. Use the specialized objects instead.
     *
     * @param $enclosedElement {jQuery} a jQuery object contained in the Form Group. It used to find the enclosing
     * HTML element having the `.form-group` class assigned.
     *
     *
     * @constructor
     */
    n.FormGroup = function ($enclosedElement) {
        if (!$enclosedElement || $enclosedElement.length === 0) {
            throw new Error("No enclosed element");
        }

        this.$formGroup = this.$findFormGroup($enclosedElement);
        this.$element = this.$formGroup.find('input, select');
        this.$label = this.$formGroup.find('.control-label');
        this.$helpBlock = this.$formGroup.find(".help-block");
    };

    n.FormGroup.prototype.isType = function (type) {
        var lowerCaseType = type.toLowerCase();
        var isOfType = true;
        this.$element.each(function () {
            if (this.hasAttribute('type')) {
                if (this.getAttribute('type') === lowerCaseType) {
                    return true;
                } else {
                    isOfType = false;
                    return false;
                }
            } else {
                // <select> is not an attribute value, obviously, so check for nodename
                if (this.nodeName.toLowerCase() === lowerCaseType) {
                    return true;
                } else if (lowerCaseType === 'text') {
                    return true;
                } else {
                    isOfType = false;
                    return false;
                }
            }
        });

        return isOfType;
    };

    /**
     *
     * @param $enclosedElement
     * @returns {*}
     *
     * @private
     */
    n.FormGroup.prototype.$findFormGroup = function ($enclosedElement) {
        var $formGroup = $enclosedElement.closest("div.form-group");

        if (!$formGroup || $formGroup.length === 0) {
            throw new Error("Unable to find Form Group");
        }

        if ($formGroup.length > 1) {
            throw new Error("enclosed element yields ambiguous form group");
        }

        return $formGroup;
    };

    n.FormGroup.prototype.hasLabel = function () {
        return this.$label.length > 0;
    };

    n.FormGroup.prototype.hasHelpBlock = function () {
        return this.$helpBlock.length > 0;
    };

    n.FormGroup.prototype.setEnabled = function (enabled) {
        this.$formGroup.data("enabled", enabled);
        this.$element.prop('disabled', !enabled);

        // Test if we're not readonly, if so set classes. This is required because we share `text-muted` with the
        // readonly-state
        if (this.$formGroup.data("readonly") !== true) {
            if (enabled) {
                this.$formGroup.removeClass("text-muted");
            } else {
                this.$formGroup.addClass("text-muted");
            }
        }

        if (enabled) {
            this.$label.removeClass("disabled");
            this.$element.parents("div[class=radio]").removeClass("disabled");
        } else {
            this.$label.addClass("disabled");
            this.$element.parents("div[class=radio]").addClass("disabled");
        }
    };

    n.FormGroup.prototype.setReadOnly = function (readonly) {
        this.$formGroup.data("readonly", readonly);
        this.$element.prop('readonly', readonly);
        this.handleReadOnlyEmulationIfRequired(readonly);

        // Test if we're enabled, if so set classes. This is required because we share `text-muted` with the
        // enabled-state
        if (this.$formGroup.data("enabled") !== false) {
            if (readonly) {
                this.$formGroup.addClass("text-muted");
            } else {
                this.$formGroup.removeClass("text-muted");
            }
        }

        if (readonly) {
            this.$label.addClass("readonly");
        } else {
            this.$formGroup.removeClass("readonly");
        }
    };

    /**
     * @private
     * @param readonlyState
     */
    n.FormGroup.prototype.handleReadOnlyEmulationIfRequired = function (readonlyState) {
        if (!this.readOnlyEmulationRequired()) {
            return;
        }

        if (readonlyState) {
            // In case we're called with readonlyState===true twice in a row, make sure only one handler will be
            // active at a time
            this.$element.off('click', this.readOnlyHandler);
            this.$element.on('click', this.readOnlyHandler);
        } else {
            this.$element.off('click', this.readOnlyHandler);
        }
    };

    n.FormGroup.prototype.readOnlyEmulationRequired = function () {
        // Keep this at top, since select does not feature the type attribute.
        if (n.readOnlyIgnored.indexOf(this.$element[0].nodeName.toLowerCase())) {
            return true;
        }

        if (!this.$element[0].hasAttribute('type')) {
            // if there is no type attribute, browsers default to text, which is `properly implements` read only.
            return false;
        }
        if (n.readOnlyIgnored.indexOf(this.$element[0].getAttribute('type').toLowerCase())) {
            return true;
        }


        return false;
    };


    /**
     * Read Only click handler.
     *
     * Since the readonly attribute does not work as expected on certain input types, emulate read only
     */
    n.FormGroup.prototype.readOnlyHandler = function () {
        return false;
    };


})(QfqNS.Element);